import rollbar
from sanic.exceptions import SanicException
from sanic.handlers import ErrorHandler

__version__ = '0.1.0'


class RollbarHandler(ErrorHandler):
    """
    A subclass of built-in Sanic exception handle, sending data to Rollbar
    """

    def default(self, request, exception):
        if not issubclass(type(exception), SanicException):
            rollbar.report_exc_info(request=request)
        return super().default(request, exception)


def _hook(request, data):
    data['framework'] = 'sanic'

    if request:
        data['context'] = str(request.url)


class SanicRollbar:
    """
    A "plugin" to automatically configure Rollbar and connect Sanic app to custom error handler.
    """

    def __init__(self, app=None, ignore_exc=None):
        self.handler = None
        self.ignore_exc = ignore_exc or []
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        ignored = [(exc, 'ignored') for exc in self.ignore_exc]
        rollbar.init(
            app.config['ROLLBAR_TOKEN'],
            environment=app.config.get('ROLLBAR_ENV', 'dev'),
            # root=os.path.dirname(os.path.realpath(__file__)),
            allow_logging_basic_config=False,
            exception_level_filters=ignored,
            # **self.init_kwargs
        )
        rollbar.BASE_DATA_HOOK = _hook

        self.handler = RollbarHandler()
        app.error_handler = self.handler


__all__ = [SanicRollbar]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import setup


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


# Does not work with tox:
# install_requires = [
#     l for l in open(join(dirname(__file__), 'requirements.txt')).read().split('\n')
#     if l and not l.startswith('#')
# ]


setup(
    name='sanic-rollbar',
    version='0.1.0',
    license='MIT license',
    description='A Rollbar plugin for Sanic',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.md')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
    ),
    author='Sergei Beilin',
    author_email='saabeilin@gmail.com',
    url='https://github.com/saabeilin/sanic-rollbar',
    # packages=find_packages('src'),
    packages=['sanic_rollbar'],
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
    ],
    keywords=[
        'sanic', 'rollbar'
    ],
    install_requires=['sanic', 'rollbar']
)
